@@ -3,7 +3,7 @@ package YAHC;
 use strict;
 use warnings;
 
-our $VERSION = '0.016';
+our $VERSION = '0.017';
 
 use EV;
 use Time::HiRes;
@@ -181,14 +181,6 @@ sub run_tick    { shift->_run(EV::RUN_NOWAIT)   }
 sub is_running  { shift->{is_running}           }
 sub loop        { shift->{loop}                 }
 
-sub active_connections {
-    my $self = shift;
-    return [ map  { $_->{id} }
-             grep { $_->{state} < YAHC::State::COMPLETED() }
-             values %{ $self->{connections} }
-    ];
-}
-
 ################################################################################
 # Routines to manipulate connections (also user facing)
 ################################################################################
@@ -307,10 +299,11 @@ sub _check_stop_condition {
 
     if ($awaiting_connections == 0) {
         $self->_break(sprintf("until state '%s' is reached", _strstate($expected_state)));
-    } else {
-        _log_message("YAHC: still have %d connections awaiting state '%s'",
-                     $awaiting_connections, _strstate($expected_state)) if $self->{debug};
+        return 1;
     }
+
+    _log_message("YAHC: still have %d connections awaiting state '%s'",
+                 $awaiting_connections, _strstate($expected_state)) if $self->{debug};
 }
 
 ################################################################################
@@ -355,12 +348,12 @@ sub _set_init_state {
             _register_in_timeline($conn, "Target $host:$port ($ip:$port) chosen for attempt #$attempt") if $conn->{keep_timeline};
 
             my $sock = _build_socket_and_connect($ip, $port, $conn->{request});
-            $self->_set_wait_synack_state($conn_id, $sock, $ip, $host, $port);
+            $self->_set_wait_synack_state($conn_id, $sock);
             $continue = 0;
             1;
         } or do {
             my $error = $@ || 'zombie error';
-            _register_error($conn, YAHC::Error::CONNECT_ERROR(), "Connection attempt #$attempt for $conn_id has failed: $error");
+            _register_error($conn, YAHC::Error::CONNECT_ERROR(), "Connection attempt failed: $error");
             #$self->_set_set_init_state_timer($conn_id); # trigger background timer which will do reconnection attempt # TODO
             #$continue = 0;
         };
@@ -368,7 +361,7 @@ sub _set_init_state {
 }
 
 sub _set_wait_synack_state {
-    my ($self, $conn_id, $sock, $ip, $host, $port) = @_;
+    my ($self, $conn_id, $sock) = @_;
 
     my $conn = $self->{connections}{$conn_id};
     my $watchers = $self->{watchers}{$conn_id};
@@ -389,7 +382,7 @@ sub _set_wait_synack_state {
 
         if (my $err = unpack("L", $sockopt)) {
             my $strerror = POSIX::strerror($err) || '<unknown POSIX error>';
-            _register_error($conn, YAHC::Error::CONNECT_ERROR(), "Failed to connect to $host:$port ($ip:$port): $strerror");
+            _register_error($conn, YAHC::Error::CONNECT_ERROR(), "Failed to connect: $strerror");
             $self->_set_init_state($conn_id);
             return;
         }
@@ -397,6 +390,7 @@ sub _set_wait_synack_state {
         $conn->{state} = YAHC::State::CONNECTED();
         _register_in_timeline($conn, "new state %s", _strstate($conn->{state})) if $conn->{keep_timeline};
         $self->_call_state_callback($conn, 'connected_callback') if $conn->{has_connected_callback};
+        return if $self->{stop_condition} && $self->_check_stop_condition($conn);
 
         $self->_set_write_state($conn_id);
     };
@@ -575,15 +569,15 @@ sub _build_socket_and_connect {
     my ($ip, $port, $timeouts) = @_;
 
     my $sock;
-    socket($sock, PF_INET, SOCK_STREAM, 0) or die "YAHC: Failed to construct TCP socket: $!\n";
+    socket($sock, PF_INET, SOCK_STREAM, 0) or die "Failed to construct TCP socket: $!\n";
 
-    my $flags = fcntl($sock, F_GETFL, 0) or die "YAHC: Failed to get fcntl F_GETFL flag: $!\n";
-    fcntl($sock, F_SETFL, $flags | O_NONBLOCK) or die "YAHC: Failed to set fcntl O_NONBLOCK flag: $!\n";
+    my $flags = fcntl($sock, F_GETFL, 0) or die "Failed to get fcntl F_GETFL flag: $!\n";
+    fcntl($sock, F_SETFL, $flags | O_NONBLOCK) or die "Failed to set fcntl O_NONBLOCK flag: $!\n";
 
-    my $ip_addr = inet_aton($ip);
+    my $ip_addr = inet_aton($ip) or die "Invalid IP address";
     my $addr = pack_sockaddr_in($port, $ip_addr);
     if (!connect($sock, $addr) && $! != EINPROGRESS) {
-        die "YAHC: Failed to connect to $ip: $!\n";
+        die "Failed to connect: $!\n";
     }
 
     return $sock;
@@ -728,7 +722,7 @@ sub _register_error {
     my ($conn, $error, $format, @arguments) = @_;
     my $strerror = sprintf("$format", @arguments);
     $strerror =~ s/\s+$//g;
-    _register_in_timeline($conn, "error=$strerror (errno=$error)") if $conn->{debug};
+    _register_in_timeline($conn, "error=$strerror (errno=$error)") if $conn->{keep_timeline};
     push @{ $conn->{errors} ||= [] }, [ $error, $strerror, [ @{ $conn->{selected_target} } ], Time::HiRes::time ];
 }
 
